/*
 * bltool - A Firmware Downloader/Uploader for ROSAND Technologies products
 * Copyright (C) 2000-2005  Brian S. Dean <bsd@bsdhome.com>
 * Copyright 2007-2009 Joerg Wunsch <j@uriah.heep.sax.de>
 * Copyright 2011, Rosand Technologies.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _8051_H_
#define _8051_H_

#include "part.h"
#include "pgm.h"

typedef void (*FP_UpdateProgress)(int percent, double etime, char *hdr);

extern struct part parts[];

extern FP_UpdateProgress update_progress;

extern char progbuf[];

extern int do_cycles;

#ifdef __cplusplus
extern "C" {
#endif

int part_read_byte_default(PROGRAMMER * pgm, PART * p, MEM * mem,
			  unsigned long addr, unsigned char * value);

int part_read(PROGRAMMER * pgm, PART * p, char * memtype, int size,
             int verbose);

int part_write_page(PROGRAMMER * pgm, PART * p, MEM * mem,
                   unsigned long addr);

int part_write_byte(PROGRAMMER * pgm, PART * p, MEM * mem,
                   unsigned long addr, unsigned char data);

int part_write_byte_default(PROGRAMMER * pgm, PART * p, MEM * mem,
			   unsigned long addr, unsigned char data);

int part_write(PROGRAMMER * pgm, PART * p, char * memtype, int size,
              int verbose);

int part_signature(PROGRAMMER * pgm, PART * p);

int part_verify(PART * p, PART * v, char * memtype, int size);

int part_get_cycle_count(PROGRAMMER * pgm, PART * p, int * cycles);

int part_put_cycle_count(PROGRAMMER * pgm, PART * p, int cycles);

long part_mem_loaddr(MEM * mem);

long part_mem_hiaddr(MEM * mem);

int part_chip_erase(PROGRAMMER * pgm, PART * p);

void report_progress (int completed, int total, char *hdr);

#ifdef __cplusplus
}
#endif

#endif /* _8051_H_ */

